<?php
/*
http://www.payline.com

Payline module for OpenCart - v1.3 - January 2014

Copyright (c) 2014 Monext
*/
require_once(DIR_SYSTEM.'payline/configuration/options.php');
require_once(DIR_SYSTEM.'payline/lib/paylineSDK.php');

class ControllerPaymentPayline extends Controller {
	
	var $paylineTrace, $errorMessage;
	
	/**
	 * @desc mapping between ISO and Payline currency codes
	 * @var $_currencies
	 */
	var $_currencies = array(
		'EUR' => '978', // Euro                                                                                                                                      
		'AFN' => '971', // Afghani                                                                                                                                   
		'ALL' => '8', // Lek                                                                                                                                         
		'DZD' => '12', // Algerian Dinar                                                                                                                             
		'USD' => '840', // US Dollar                                                                                                                                 
		'AOA' => '973', // Kwanza                                                                                                                                    
		'XCD' => '951', // East Caribbean Dollar                                                                                                                     
		'ARS' => '32', // Argentine Peso                                                                                                                             
		'AMD' => '51', // Armenian Dram                                                                                                                              
		'AWG' => '533', // Aruban Guilder                                                                                                                            
		'AUD' => '36', // Australian Dollar                                                                                                                          
		'AZN' => '944', // Azerbaijanian Manat                                                                                                                       
		'BSD' => '44', // Bahamian Dollar                                                                                                                            
		'BHD' => '48', // Bahraini Dinar                                                                                                                             
		'BDT' => '50', // Taka                                                                                                                                       
		'BBD' => '52', // Barbados Dollar                                                                                                                            
		'BYR' => '974', // Belarussian Ruble                                                                                                                         
		'BZD' => '84', // Belize Dollar                                                                                                                              
		'XOF' => '952', // CFA Franc BCEAO                                                                                                                          
		'BMD' => '60', // Bermudian Dollar (customarily known as Bermuda Dollar)                                                                                     
		'INR' => '356', // Indian Rupee                                                                                                                              
		'BTN' => '64', // Ngultrum                                                                                                                                   
		'BOB' => '68', // Boliviano                                                                                                                                  
		'BOV' => '984', // Mvdol                                                                                                                                     
		'BAM' => '977', // Convertible Marks                                                                                                                         
		'BWP' => '72', // Pula                                                                                                                                       
		'NOK' => '578', // Norwegian Krone                                                                                                                           
		'BRL' => '986', // Brazilian Real                                                                                                                            
		'BND' => '96', // Brunei Dollar                                                                                                                              
		'BGN' => '975', // Bulgarian Lev                                                                                                                             
		'BIF' => '108', // Burundi Franc                                                                                                                             
		'KHR' => '116', // Riel                                                                                                                                      
		'XAF' => '950', // CFA Franc BEAC                                                                                                                           
		'CAD' => '124', // Canadian Dollar                                                                                                                           
		'CVE' => '132', // Cape Verde Escudo                                                                                                                         
		'KYD' => '136', // Cayman Islands Dollar                                                                                                                     
		'CLP' => '152', // Chilean Peso                                                                                                                              
		'CLF' => '990', // Unidades de formento                                                                                                                      
		'CNY' => '156', // Yuan Renminbi                                                                                                                             
		'COP' => '170', // Colombian Peso                                                                                                                            
		'COU' => '970', // Unidad de Valor Real                                                                                                                      
		'KMF' => '174', // Comoro Franc                                                                                                                              
		'CDF' => '976', // Franc Congolais                                                                                                                           
		'NZD' => '554', // New Zealand Dollar                                                                                                                        
		'CRC' => '188', // Costa Rican Colon                                                                                                                         
		'HRK' => '191', // Croatian Kuna                                                                                                                             
		'CUP' => '192', // Cuban Peso                                                                                                                                
		'CYP' => '196', // Cyprus Pound                                                                                                                              
		'CZK' => '203', // Czech Koruna                                                                                                                              
		'DKK' => '208', // Danish Krone                                                                                                                              
		'DJF' => '262', // Djibouti Franc                                                                                                                            
		'DOP' => '214', // Dominican Peso                                                                                                                            
		'EGP' => '818', // Egyptian Pound                                                                                                                            
		'SVC' => '222', // El Salvador Colon                                                                                                                         
		'ERN' => '232', // Nakfa                                                                                                                                     
		'EEK' => '233', // Kroon                                                                                                                                     
		'ETB' => '230', // Ethiopian Birr                                                                                                                            
		'FKP' => '238', // Falkland Islands Pound                                                                                                                    
		'FJD' => '242', // Fiji Dollar                                                                                                                               
		'XPF' => '953', // CFP Franc                                                                                                                                 
		'GMD' => '270', // Dalasi                                                                                                                                    
		'GEL' => '981', // Lari                                                                                                                                      
		'GHC' => '288', // Cedi                                                                                                                                      
		'GIP' => '292', // Gibraltar Pound                                                                                                                           
		'GTQ' => '320', // Quetzal                                                                                                                                   
		'GNF' => '324', // Guinea Franc                                                                                                                              
		'GWP' => '624', // Guinea-Bissau Peso                                                                                                                        
		'GYD' => '328', // Guyana Dollar                                                                                                                             
		'HTG' => '332', // Gourde                                                                                                                                    
		'HNL' => '340', // Lempira                                                                                                                                   
		'HKD' => '344', // Hong Kong Dollar                                                                                                                          
		'HUF' => '348', // Forint                                                                                                                                    
		'ISK' => '352', // Iceland Krona                                                                                                                             
		'IDR' => '360', // Rupiah                                                                                                                                    
		'XDR' => '960', // SDR                                                                                                                                       
		'IRR' => '364', // Iranian Rial                                                                                                                              
		'IQD' => '368', // Iraqi Dinar                                                                                                                               
		'ILS' => '376', // New Israeli Sheqel                                                                                                                        
		'JMD' => '388', // Jamaican Dollar                                                                                                                           
		'JPY' => '392', // Yen                                                                                                                                       
		'JOD' => '400', // Jordanian Dinar                                                                                                                           
		'KZT' => '398', // Tenge                                                                                                                                     
		'KES' => '404', // Kenyan Shilling                                                                                                                           
		'KPW' => '408', // North Korean Won                                                                                                                          
		'KRW' => '410', // Won                                                                                                                                       
		'KWD' => '414', // Kuwaiti Dinar                                                                                                                             
		'KGS' => '417', // Som                                                                                                                                       
		'LAK' => '418', // Kip                                                                                                                                       
		'LVL' => '428', // Latvian Lats                                                                                                                              
		'LBP' => '422', // Lebanese Pound                                                                                                                            
		'ZAR' => '710', // Rand                                                                                                                                      
		'LSL' => '426', // Loti                                                                                                                                      
		'LRD' => '430', // Liberian Dollar                                                                                                                           
		'LYD' => '434', // Libyan Dinar                                                                                                                              
		'CHF' => '756', // Swiss Franc                                                                                                                               
		'LTL' => '440', // Lithuanian Litas                                                                                                                          
		'MOP' => '446', // Pataca                                                                                                                                    
		'MKD' => '807', // Denar                                                                                                                                     
		'MGA' => '969', // Malagascy Ariary                                                                                                                          
		'MWK' => '454', // Kwacha                                                                                                                                    
		'MYR' => '458', // Malaysian Ringgit                                                                                                                         
		'MVR' => '462', // Rufiyaa                                                                                                                                   
		'MTL' => '470', // Maltese Lira                                                                                                                              
		'MRO' => '478', // Ouguiya                                                                                                                                   
		'MUR' => '480', // Mauritius Rupee                                                                                                                           
		'MXN' => '484', // Mexican Peso                                                                                                                              
		'MXV' => '979', // Mexican Unidad de Inversion (UID)                                                                                                         
		'MDL' => '498', // Moldovan Leu                                                                                                                              
		'MNT' => '496', // Tugrik                                                                                                                                    
		'MAD' => '504', // Moroccan Dirham                                                                                                                           
		'MZN' => '943', // Metical                                                                                                                                   
		'MMK' => '104', // Kyat                                                                                                                                      
		'NAD' => '516', // Namibian Dollar                                                                                                                           
		'NPR' => '524', // Nepalese Rupee                                                                                                                            
		'ANG' => '532', // Netherlands Antillian Guilder                                                                                                             
		'NIO' => '558', // Cordoba Oro                                                                                                                               
		'NGN' => '566', // Naira                                                                                                                                     
		'OMR' => '512', // Rial Omani                                                                                                                                
		'PKR' => '586', // Pakistan Rupee                                                                                                                            
		'PAB' => '590', // Balboa                                                                                                                                    
		'PGK' => '598', // Kina                                                                                                                                      
		'PYG' => '600', // Guarani                                                                                                                                   
		'PEN' => '604', // Nuevo Sol                                                                                                                                 
		'PHP' => '608', // Philippine Peso                                                                                                                           
		'PLN' => '985', // Zloty                                                                                                                                     
		'QAR' => '634', // Qatari Rial                                                                                                                               
		'ROL' => '642', // Old Leu                                                                                                                                   
		'RON' => '946', // New Leu                                                                                                                                   
		'RUB' => '643', // Russian Ruble                                                                                                                             
		'RWF' => '646', // Rwanda Franc                                                                                                                              
		'SHP' => '654', // Saint Helena Pound                                                                                                                        
		'WST' => '882', // Tala                                                                                                                                      
		'STD' => '678', // Dobra                                                                                                                                     
		'SAR' => '682', // Saudi Riyal                                                                                                                               
		'RSD' => '941', // Serbian Dinar                                                                                                                             
		'SCR' => '690', // Seychelles Rupee                                                                                                                          
		'SLL' => '694', // Leone                                                                                                                                     
		'SGD' => '702', // Singapore Dollar                                                                                                                          
		'SKK' => '703', // Slovak Koruna                                                                                                                             
		'SIT' => '705', // Tolar                                                                                                                                     
		'SBD' => '90', // Solomon Islands Dollar                                                                                                                     
		'SOS' => '706', // Somali Shilling                                                                                                                           
		'LKR' => '144', // Sri Lanka Rupee                                                                                                                           
		'SDG' => '938', // Sudanese Dinar                                                                                                                            
		'SRD' => '968', // Surinam Dollar                                                                                                                            
		'SZL' => '748', // Lilangeni                                                                                                                                 
		'SEK' => '752', // Swedish Krona                                                                                                                             
		'CHW' => '948', // WIR Franc                                                                                                                                 
		'CHE' => '947', // WIR Euro                                                                                                                                  
		'SYP' => '760', // Syrian Pound                                                                                                                              
		'TWD' => '901', // New Taiwan Dollar                                                                                                                         
		'TJS' => '972', // Somoni                                                                                                                                    
		'TZS' => '834', // Tanzanian Shilling                                                                                                                        
		'THB' => '764', // Baht                                                                                                                                      
		'TOP' => '776', // Pa'anga                                                                                                                                   
		'TTD' => '780', // Trinidad and Tobago Dollar                                                                                                                
		'TND' => '788', // Tunisian Dinar                                                                                                                            
		'TRY' => '949', // New Turkish Lira                                                                                                                          
		'TMM' => '795', // Manat                                                                                                                                     
		'UGX' => '800', // Uganda Shilling                                                                                                                           
		'UAH' => '980', // Hryvnia                                                                                                                                   
		'AED' => '784', // UAE Dirham                                                                                                                                
		'GBP' => '826', // Pound Sterling                                                                                                                            
		'USS' => '998', // (Same day)                                                                                                                                
		'USN' => '997', // (Next day)                                                                                                                                
		'UYU' => '858', // Peso Uruguayo                                                                                                                             
		'UYI' => '940', // Uruguay Peso en Unidades Indexadas                                                                                                        
		'UZS' => '860', // Uzbekistan Sum                                                                                                                            
		'VUV' => '548', // Vatu                                                                                                                                      
		'VEB' => '862', // Bolivar                                                                                                                                   
		'VND' => '704', // Dong                                                                                                                                      
		'YER' => '886', // Yemeni Rial                                                                                                                               
		'ZMK' => '894', // Kwacha                                                                                                                                    
		'ZWD' => '716', // Zimbabwe Dollar                                                                                                                           
		'XAU' => '959', // Gold                                                                                                                                      
		'XBA' => '955', // Bond Markets Units European Composite Unit (EURCO)                                                                                        
		'XBB' => '956', // European Monetary Unit (E.M.U.-6)                                                                                                         
		'XBC' => '957', // European Unit of Account 9(E.U.A.-9)                                                                                                      
		'XBD' => '958', // European Unit of Account 17(E.U.A.-17)                                                                                                    
		'XPD' => '964', // Palladium                                                                                                                                 
		'XPT' => '962', // Platinum                                                                                                                                  
		'XAG' => '961', // Silver                                                                                                                                    
		'XTS' => '963', // Codes specifically reserved for testing purposes                                                                                          
		'XXX' => '999', // The codes assigned for transactions where no currency is involved 
	);
	
	/**
	* @method index
	* @desc Initialize and display Payline order confirm button
	*/
	protected function index() {
		$this->load->model('checkout/order');
		$this->language->load('payment/payline');
		$this->data['button_confirm'] = $this->language->get('button_confirm');
		$this->data['action'] = HTTP_SERVER.'index.php?route=payment/payline/doWebPayment';
		$this->data['production'] = $this->config->get('production');
		$this->data['text_homologation'] = $this->language->get('text_homologation');
		
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/payline.tpl')) {
			$this->template = $this->config->get('config_template') . '/template/payment/payline.tpl';
		} else {
			$this->template = 'default/template/payment/payline.tpl';
		}
		$this->render();
	}
	
	/**
	 * @method formatToUtf8
	 * @desc returns an UTF-8 version of the input string
	 * @param $inString : the string to format
	 */
	private function formatToUtf8($inString){
		$cur_encoding = mb_detect_encoding($inString) ;
		if($cur_encoding == "UTF-8" && mb_check_encoding($inString,"UTF-8")){
			$outString = $inString;
		}else{
			$outString = utf8_encode($inString);
		}
		return $outString;
	}
	
	/**
	 * @method writeTrace
	 * @desc write a trace in Payline log file
	 * @param $trace : the string to add in the log file
	 */
	private function writeTrace($trace){
		if(!isset($this->paylineTrace)){
			$this->paylineTrace = new Log('payline_'.date('Y-m-d',time()).'.txt');
		}
		$this->paylineTrace->write($trace);
	}
	
	/**
	* @method callback
	* @desc this function is called after payment, when the customer comes back to the store.
	* It retrieves the payment result and update the order.
	**/
	public function callback() {
		$this->language->load('payment/payline');
		
		$paylineSDK = new paylineSDK(
			$this->config->get('payline_merchant_id'),
			$this->config->get('payline_access_key'),
			$this->config->get('proxy_host'),
			$this->config->get('proxy_port'),
			$this->config->get('proxy_login'),
			$this->config->get('proxy_password'),
			$this->config->get('production')
		);

		$getWebPaymentDetailsParam = array('version' => 2, 'token' => $_GET['token']);
		$getWebPaymentDetailsResponse = $paylineSDK->getWebPaymentDetails($getWebPaymentDetailsParam);
		if(isset($getWebPaymentDetailsResponse) && isset($getWebPaymentDetailsResponse['result']['code'])){
			$getWebPaymentDetailsResponseCode = $getWebPaymentDetailsResponse['result']['code'];
			$order_id = $getWebPaymentDetailsResponse['order']['ref'];
			$this->load->model('checkout/order');
			
			if($getWebPaymentDetailsResponseCode == '00000'){
				$comment = $this->language->get('text_transaction').$getWebPaymentDetailsResponse['transaction']['id'];
				if(isset($getWebPaymentDetailsResponse['card'])){
					if(isset($getWebPaymentDetailsResponse['card']['number'])){
						$comment.= '<br/>'.$this->language->get('text_card_number').$getWebPaymentDetailsResponse['card']['number'];
					}
					if(isset($getWebPaymentDetailsResponse['card']['type'])){
						$comment.= '<br/>'.$this->language->get('text_card_type').$getWebPaymentDetailsResponse['card']['type'];
					}
					if(isset($getWebPaymentDetailsResponse['card']['expirationDate'])){
						$comment.= '<br/>'.$this->language->get('text_card_exp').$getWebPaymentDetailsResponse['card']['expirationDate'];
					}
				}
				$this->model_checkout_order->update($order_id, $this->config->get('accepted_status_id'), $comment, true);
				$this->writeTrace("order $order_id - transaction OK : ".$getWebPaymentDetailsResponse['transaction']['id']);
				
				if($_GET['action'] != 'notif'){
					$this->writeTrace('order '.$order_id.' - action='.$_GET['action'].' - redirection to '.HTTP_SERVER.'index.php?route=checkout/success');
					$this->redirect(HTTP_SERVER.'index.php?route=checkout/success');
				}else {
					$this->writeTrace('order '.$order_id.' - action= '.$_GET['action']);
					if (isset($this->session->data['order_id'])) {
						$this->cart->clear();
						unset($this->session->data['shipping_method']);
						unset($this->session->data['shipping_methods']);
						unset($this->session->data['payment_method']);
						unset($this->session->data['payment_methods']);
						unset($this->session->data['guest']);
						unset($this->session->data['comment']);
						unset($this->session->data['order_id']);
						unset($this->session->data['coupon']);
						unset($this->session->data['reward']);
						unset($this->session->data['voucher']);
						unset($this->session->data['vouchers']);
						$this->writeTrace('order '.$order_id.' - cart reset OK');
					}else{
						$this->writeTrace('order '.$order_id.' - unable to reset cart');
					}
				}
				exit;
			}else{
				$comment = $getWebPaymentDetailsResponse['result']['shortMessage']." (code $getWebPaymentDetailsResponseCode) : ".$getWebPaymentDetailsResponse['result']['longMessage'];
				$this->writeTrace('order '.$order_id.' - '.$comment);
				
				switch($getWebPaymentDetailsResponseCode) {
					case '02319':
						$this->model_checkout_order->update($order_id, $this->config->get('canceled_status_id'), $comment, true);
						break;
					case '02304':     
					case '02324':
					case '02534':
						$this->model_checkout_order->update($order_id, $this->config->get('abandoned_status_id'), $comment, true);
						break;
          case '02306':
          case '02533':
            exit;
            break;
					default:
						$this->model_checkout_order->update($order_id, $this->config->get('refused_status_id'), $comment, true);
						break;
				}
				if($_GET['action'] != 'notif'){
					$this->display_error_page(2,$comment);
				}
			}
		}else{
			$this->model_checkout_order->update($order_id, $this->config->get('pending_status_id'), $getWebPaymentDetailsResponse, true);
			$this->writeTrace($getWebPaymentDetailsResponse);
			if($_GET['action'] != 'notif'){
				$this->display_error_page(2,$getWebPaymentDetailsResponse);
			}
		}
	}
	
	/**
	* @method doWebPayment
	* @desc prepare parameters and call Payline doWebPayment web service
	**/
	public function doWebPayment() {
		$this->language->load('payment/payline');
		
		$this->load->model('checkout/order');
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
		
		$paylineSDK = new paylineSDK(
			$this->config->get('payline_merchant_id'),
			$this->config->get('payline_access_key'),
			$this->config->get('proxy_host'),
			$this->config->get('proxy_port'),
			$this->config->get('proxy_login'),
			$this->config->get('proxy_password'),
			$this->config->get('production')
		);
		
		$paylineSDK->returnURL = HTTP_SERVER.'index.php?route=payment/payline/callback&action=return';
		$paylineSDK->cancelURL = HTTP_SERVER.'index.php?route=payment/payline/callback&action=cancel';
		$paylineSDK->notificationURL = HTTP_SERVER.'index.php?route=payment/payline/callback&action=notif';
		
		$payline_primary_contracts = $this->config->get('payline_primary_contracts');
		$payline_secondary_contracts = $this->config->get('payline_secondary_contracts');
		$doWebPaymentParams = array();
		$doWebPaymentParams['payment']['action'] = $this->config->get('payment_action');
		$doWebPaymentParams['payment']['mode'] = 'CPT';
		$doWebPaymentParams['payment']['amount'] = round($order_info['total']*$order_info['currency_value']*100);
		$doWebPaymentParams['payment']['currency'] = $this->_currencies[$order_info['currency_code']];
		$doWebPaymentParams['payment']['contractNumber'] = $payline_primary_contracts[0];
		
		$doWebPaymentParams['contracts'] = $payline_primary_contracts;
		$doWebPaymentParams['secondContracts'] = $payline_secondary_contracts;
		
		$doWebPaymentParams['customPaymentTemplateURL'] = $this->config->get('template_url');
		$doWebPaymentParams['customPaymentPageCode'] = $this->config->get('custom_page_code');
		
		$doWebPaymentParams['order']['ref'] = $order_info['order_id'];
		$doWebPaymentParams['order']['amount'] = $doWebPaymentParams['payment']['amount'];
		$doWebPaymentParams['order']['currency'] =  $doWebPaymentParams['payment']['currency'];
		$doWebPaymentParams['order']['origin'] = 1 ;
		$doWebPaymentParams['order']['country'] = $order_info['payment_iso_code_2'];
		$doWebPaymentParams['order']['date'] = date('d/m/Y H:i',time());
		
		foreach ($this->cart->getProducts() as $product) {
			$order_detail = array();
			$order_detail['ref'] = $product['product_id'];
			$order_detail['price'] = round($product['price']*$order_info['currency_value']*100);
			$order_detail['quantity'] = $product['quantity'];
			$order_detail['comment'] = $product['name'];
			$paylineSDK->setItem($order_detail);
		}
		
		$doWebPaymentParams['buyer']['lastName'] = $this->formatToUtf8($order_info['payment_lastname']);
		$doWebPaymentParams['buyer']['firstName'] = $this->formatToUtf8($order_info['payment_firstname']);
		$doWebPaymentParams['buyer']['email'] = $order_info['email'];
		$doWebPaymentParams['buyer']['ip'] = $order_info['ip'];
		
		$doWebPaymentParams['address']['name'] = $doWebPaymentParams['buyer']['firstName'] . ' ' . $doWebPaymentParams['buyer']['lastName'];
		$doWebPaymentParams['address']['street1'] = $order_info['shipping_address_1'];
		$doWebPaymentParams['address']['street2'] = $order_info['shipping_address_2'];;
		$doWebPaymentParams['address']['cityName'] =  $order_info['shipping_city'];
		$doWebPaymentParams['address']['zipCode'] =  $order_info['shipping_postcode'];
		$doWebPaymentParams['address']['country'] = $order_info['shipping_country'];
		$forbidenCars = array(' ','.','(',')','-');
		$doWebPaymentParams['address']['phone'] = str_replace($forbidenCars,'',$order_info['telephone']);
		
		$doWebPaymentResponse = $paylineSDK->doWebPayment($doWebPaymentParams);
		
		if(isset($doWebPaymentResponse) && isset($doWebPaymentResponse['result']['code'])){
			if($doWebPaymentResponse['result']['code'] == '00000'){
				$this->writeTrace("order ".$order_info['order_id']." - doWebPayment OK : token ".$doWebPaymentResponse['token']);
				$this->model_checkout_order->confirm($order_info['order_id'], $this->config->get('init_status_id'), $this->language->get('text_token').$doWebPaymentResponse['token'], true);
				$this->redirect($doWebPaymentResponse['redirectURL']);
				exit();
			}else{
				$this->errorMessage = $doWebPaymentResponse['result']['shortMessage'].' (code '.$doWebPaymentResponse['result']['code'].') : '.$doWebPaymentResponse['result']['longMessage'];
			}
		}else{
			$this->errorMessage = $doWebPaymentResponse;
		}
		$this->writeTrace("order ".$order_info['order_id']." - ".$this->errorMessage);
		$this->display_error_page(1,$this->errorMessage);
	}
	
	/**
	* @method display_error_page
	* @param $type : 1 => error at Payline call, 2 => payment error
	* @param $errorMessage : message to be displayed
	* @desc display an error page to the customer
	**/
	public function display_error_page($type,$errorMessage){
		$this->language->load('payment/payline');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->data['breadcrumbs'] = array();
		$this->data['breadcrumbs'][] = array(
		        	'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home'),
		        	'separator' => false
		);
		
		if($type == 1){
			$this->data['heading_title'] = $this->language->get('payline_error');			
		}else if ($type == 2){
			$this->data['heading_title'] = $this->language->get('payment_refused');
		}
		$this->data['text_error'] = $errorMessage;
		$this->data['button_continue'] = $this->language->get('button_continue');
		$this->data['continue'] = $this->url->link('common/home');
		
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/error/payline.tpl')) {
			$this->template = $this->config->get('config_template') . '/template/error/payline.tpl';
		} else {
			$this->template = 'default/template/error/not_found.tpl';
		}
		
		$this->children = array(
					'common/column_left',
					'common/column_right',
					'common/content_top',
					'common/content_bottom',
					'common/footer',
					'common/header'
		);
		
		$this->response->setOutput($this->render());
	}
}
?>